#!/usr/bin/python
import os
import sys
import struct
import time

import speex
import horde_api
import asyncio
import alsaaudio

#
# global constants
#
HORDE_SESSION_NAME = "audio"
HORDE_STREAM_NAME = "audio_stream"

MIC_SAMPLE_RATE = 8000		# sampling for sent audio
MIC_CHANNELS = 1       				# number of channels in sent audio
MIC_AUDIO_FORMAT = alsaaudio.PCM_FORMAT_S16_LE	# format of the input audio
MIC_PERIOD_SIZE = 100				# no. of samples we want to read

SPEEX_QUALITY_LEVEL = 5  # encoding quality: 0 is worst, 8 is best


#
# parse command line
#
if len(sys.argv) != 2:
    print "usage: ", sys.argv[0],"<destination host> 2> debug.log"
    sys.exit(2)

dest_host = sys.argv[1]

#
# initialize speex
#
speex_encoder = speex.new(SPEEX_QUALITY_LEVEL)

#
# initialize the mic input
#
mic_input = alsaaudio.PCM(alsaaudio.PCM_CAPTURE,alsaaudio.PCM_NORMAL)
mic_input.setchannels(MIC_CHANNELS)
mic_input.setrate(MIC_SAMPLE_RATE)
mic_input.setformat(MIC_AUDIO_FORMAT)
mic_input.setperiodsize(MIC_PERIOD_SIZE)


#
# miscellaneous
#
start_time = time.time()

def debug_timestamp():
    t = time.time() - start_time
    t *= 1000
    return "%8d\t" % int(t)

#
# horde-related globals and helper functions
#
hstream = 0
last_rtt = -1
send_rate = 0.0

def session_cb(x):
    if (x == horde_api.EVENT_SESSION_CLOSED):
        print "session closed. exiting.\n"
        horde_api.session_close(HORDE_SESSION_NAME)
        sys.exit(0)

def event_cb(event, stream_handle, a, b, c, d):
    global last_rtt
    if (event == horde_api.EVENT_ADU_FRAG_ACK):
        sys.stderr.write(debug_timestamp() + "ACK\t%d ms\n" % d)
        if (last_rtt < 32000): last_rtt = d    

def recv_cb(stream_handle, adu_id, totlen, offset, data):
    # simplex stream: should never receive data
    return

def rate_to_kbps(rate):
    kbps = (rate * 8.0) / 1024;
    return "%.1f kbps" % kbps

def stat_cb(total_send, total_recv, total_allowed_send, nchannels):
    global send_rate
    send_rate = (total_send * 8.0) / 1024;
    sys.stderr.write("statistics: send=%d (%s), allowed=%d (%s), nchannels=%d\n"\
          % (total_send, send_rate, total_allowed_send, rate_to_kbps(total_allowed_send), nchannels))

def horde_event_loop(time):
    while (horde_api.event_dispatch(time) > 0):
        pass

#
# sending data
#
adu_id = 0

def get_next_adu():
    """constructs the next adu using decoded_frames and last_rtt
       and returns (adu_id, adu_data), or (None, None)"""
    global hstream, adu_id, last_rtt

    # get rtt
    rtt_short = struct.pack("h", last_rtt)
    last_rtt = -1 # only send rtt once

    # get input samples for adu payload
    read_size, audio_data = mic_input.read()
    print "Read size: \t %0.3d \t len Audio Data: %d" % (read_size, len(audio_data))

    # encode adu payload
    encoded_data = speex_encoder.encode(audio_data, True)

    # return adu
    adu = rtt_short + encoded_data
    adu_id += 1

    return (adu_id, adu)
    
#
# I/O event handlers
# 


def capture_and_send_audio():
    """
    Called when it is time to encode and send another frame.
    """
    adu_id, adu = get_next_adu()

    if adu_id:
        horde_api.stream_send(hstream, adu_id, adu, fragment_allowed=0)
        sys.stderr.write(debug_timestamp() + "SENT\t%d\t%d\n" % (adu_id, len(adu)))
                
#
# main program loop
#
if (horde_api.session_connect(dest_host, HORDE_SESSION_NAME)):
        print "connected to server '%s'." % dest_host
        h = horde_api.stream_connect(HORDE_STREAM_NAME)
        if (h > 0):
            hstream = h
            print "\tstream '%s' connected, uid:%d." % (HORDE_STREAM_NAME, h)
            
            # register callbacks
            horde_api.session_set_callback(session_cb)
            horde_api.stream_set_event_callback(h, event_cb)
            horde_api.stream_set_recv_callback(h, recv_cb)
            horde_api.statistics_set_callback(stat_cb)
            horde_api.statistics_enable(1)

	    while True:
		capture_and_send_audio()
		while horde_api.event_dispatch(0) > 0:
			pass
            
            horde_api.session_close(HORDE_SESSION_NAME)
