/** @file stats.h
 * Functions to manage statistics reporting.
 * 
 * This the only code afik that violates the syscall/session/core/net
 * layering. A timer fires every second causing the generation of an
 * ippcmsg_statistics object, using the channel_statistics interface
 * to the net layer. The generated IPPC message is sent to every
 * ippc_handler that has registered for statistics reports.
 */

#include "syscall/syscall.h"

HORDE_NAMESPACE_BEGIN;

/** 
 * Registers a statistics listener: statistics report messages will be
 * periodically sent to the given handler.
 */
void add_statistics_listener(ippc_handler* handler);

/** 
 * Removes a statistics listener if it was previously added :
 * statistics report messages will no longer be sent to the given
 * handler.
 */
void remove_statistics_listener(ippc_handler* handler);

HORDE_NAMESPACE_END;
