// -*- mode: C++; -*-

/** @file stats.h
 *
 * The channel statistics abstraction exported to the core.
 */

#ifndef __HORDE_CHANNEL_STATS_H__
#define __HORDE_CHANNEL_STATS_H__

#include "horde_namespace.h"

HORDE_NAMESPACE_BEGIN;

/**
 * An interface to access channel statistics.
 */
class channel_statistics {
public:
  /// Gets a string identifier (unique string per channel).
  virtual string get_channel_name() = 0;

  /// Returns smoothed/averaged round-trip-time (in msecs)
  virtual int get_average_rtt_msec() = 0;
  
  /// Returns the near-past loss rate, as a percentage between 0 and 100.
  virtual int get_loss_rate() = 0;
  
  /// Returns near-past average send rate, in bytes/sec
  virtual int get_average_send_rate() = 0;
  
  /// Returns near-past average recv rate, in bytes/sec
  virtual int get_average_recv_rate() = 0;
  
  /**
   * Returns an estimate for the  allowed send rate, in bytes/sec.
   *
   * The real rate will be given by the sequence of TX slot generations,
   * but this provides an estimate the scheduler/bwallocator can use.
   */
  virtual int get_allowed_send_rate() = 0;
};

HORDE_NAMESPACE_END;

#endif
