// -*- mode: C++; -*-

/*
 * NMSTL, the Networking, Messaging, Servers, and Threading Library for C++
 * Copyright (c) 2002 Massachusetts Institute of Technology
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef NMSTL_H_NMSTL
#define NMSTL_H_NMSTL

#define NMSTL_VERSION "0.5.asfandyar"

/** \file
 * Internal declarations.
 */

#define NMSTL_NAMESPACE_BEGIN namespace std {}; namespace nmstl { using namespace ::std;
#define NMSTL_NAMESPACE_END };

#define NMSTL_ARITH_OPS(T, m) \
bool operator == (const T& x) const { return m == x.m; } \
bool operator != (const T& x) const { return m != x.m; } \
bool operator < (const T& x) const { return m < x.m; } \
bool operator > (const T& x) const { return m > x.m; } \
bool operator <= (const T& x) const { return m <= x.m; } \
bool operator >= (const T& x) const { return m >= x.m; }

#define NMSTL_COMPARE_OPS(T, m) NMSTL_ARITH_OPS(T, m)

#define NMSTL_COMPARE_OPS_3(T, m1, m2, m3) \
bool operator == (const T& x) const { return m1 == x.m1 && m2 == x.m2 && m3 == x.m3; } \
bool operator < (const T& x) const { \
    return \
        m1 < x.m1 ? 1 : \
        !(m1 == x.m1) ? 0 : \
        m2 < x.m2 ? 1 : \
        !(m2 == x.m2) ? 0 : \
        m3 < x.m3 ? 1 : \
        0; \
}

#define NMSTL_ARITH_OPS_T1(T, U, m) \
template<typename T> bool operator == (const U& x) const { return m == x.m; } \
template<typename T> bool operator != (const U& x) const { return m != x.m; } \
template<typename T> bool operator < (const U& x) const { return m < x.m; } \
template<typename T> bool operator > (const U& x) const { return m > x.m; } \
template<typename T> bool operator <= (const U& x) const { return m <= x.m; } \
template<typename T> bool operator >= (const U& x) const { return m >= x.m; }

#define NMSTL_COMPARE_OPS_T1(T, U, m) NMSTL_ARITH_OPS_T1(T, U, m)

#define NMSTL_TO_STRING_INTL(T)
#define NMSTL_TO_STRING(T)
#define NMSTL_TO_STRING_INTL_T1(T)
#define NMSTL_TO_STRING_T1(T)
#define NMSTL_TO_STRING_INTL_T2(T)
#define NMSTL_TO_STRING_T2(T)

#include <iostream>
#include <iomanip>
#include <string>
#include <sstream>
#include <cstdio>
#include <typeinfo>
#include <map>
#include <vector>
#include <list>

#include <cxxabi.h>

NMSTL_NAMESPACE_BEGIN;



template<class CharT, class Traits, class Alloc, class T>
inline basic_string<CharT, Traits, Alloc> operator + (const basic_string<CharT, Traits, Alloc>& t, const T& u) {
    return t + to_string(u);
}

template<class CharT, class Traits, class Alloc>
inline basic_string<CharT, Traits, Alloc> operator + (const basic_string<CharT, Traits, Alloc>& t, char *u) {
    return t + static_cast<const char*>(u);
}

template<class CharT, class Traits, class Alloc, class T>
inline basic_string<CharT, Traits, Alloc>& operator << (basic_string<CharT, Traits, Alloc>& t, const T& u) {
    return t += to_string(u);
}

template<class CharT, class Traits, class Alloc, class T>
inline basic_string<CharT, Traits, Alloc>& operator += (basic_string<CharT, Traits, Alloc>& t, const T& u) {
    return t += to_string(u);
}

template<class CharT, class Traits, class T>
inline basic_ostream<CharT, Traits>& operator << (basic_ostream<CharT, Traits>& out, const T& u) {
    return out << to_string(u);
}



#define NMSTL_TO_STRING_STREAM(A)
#define NMSTL_TO_STRING_STREAM_T1(A)
#define NMSTL_TO_STRING_STREAM_T2(A,B)


template<class T>
inline string to_string(const T& t) {
    return t.as_string();
}

template<class T>
inline string to_string(const T* t) {
    char buf[64];
    sprintf(buf, "&(%p)", t);
    return buf;
}
template<class T>
inline string to_string(T* t) {
    char buf[64];
    sprintf(buf, "&(%p)", t);
    return buf;
}

inline string to_string(const char* ch) {
    if (!ch) return "(null)";
    return string(ch);
}
inline string to_string(char* ch) {
    if (!ch) return "(null)";
    return string(ch);
}

inline string to_string(const string& s) {
    return s;
}

inline string to_string(const void* t) {
    char buf[64];
    sprintf(buf, "%p", t);
    return buf;
}
inline string to_string(void* t) {
    char buf[64];
    sprintf(buf, "%p", t);
    return buf;
}

inline string to_string(const bool& i) {
    static string t = "true";
    static string f = "false";
    return i ? t : f;
}

template<typename ConstIterator>
inline string to_string(ConstIterator begin, ConstIterator end) {
    string out;
    out.append("{");

    bool first = true;

    while (begin != end) {
        if (first)
            first = false;
        else
            out.append(", ");

        out.append(to_string(*begin));
        ++begin;
    }

    out.append("}");

    return out;
}

template<typename T, typename U>
inline string to_string(const map<T,U>& m) {
    string out = "{";

    for (typename map<T, U>::const_iterator i = m.begin(); i != m.end(); ++i) {
        if (i != m.begin()) out.append("; ");
        out.append(to_string(i->first)).append("->").append(to_string(i->second));
    }

    out.append("}");
    return out;
}

template<typename T>
inline string to_string(const vector<T>& m) {
    return to_string(m.begin(), m.end());
}

template<typename T>
inline string to_string(const list<T>& m) {
    return to_string(m.begin(), m.end());
}

template<typename T, typename U>
inline string to_string(const pair<T,U>& m) {
    return "(" + to_string(m.first) + "," + to_string(m.second) + ")";
}

string demangle(const char* type);
inline string demangle(const string& type) { return demangle(type.c_str()); }
inline string to_string(const type_info& i) { return demangle(i.name()); }

#define NMSTL_SCALAR_TO_STRING(T) \
inline string to_string(T t) { ostringstream o; o << t; return o.str(); }

NMSTL_SCALAR_TO_STRING(char);
NMSTL_SCALAR_TO_STRING(unsigned char);
NMSTL_SCALAR_TO_STRING(short);
NMSTL_SCALAR_TO_STRING(unsigned short);
NMSTL_SCALAR_TO_STRING(int);
NMSTL_SCALAR_TO_STRING(unsigned int);
NMSTL_SCALAR_TO_STRING(long);
NMSTL_SCALAR_TO_STRING(unsigned long);
NMSTL_SCALAR_TO_STRING(long long);
NMSTL_SCALAR_TO_STRING(unsigned long long);

inline string to_string(const float& t) { ostringstream o; o << setprecision(99) << t; return o.str(); }
inline string to_string(const double& t) { ostringstream o; o << setprecision(99) << t; return o.str(); }

template<typename T>
inline bool to_scalar(string s, T& t) {
    istringstream i(s);
    if (i >> t) {
        return true;
    } else {
        t = T();
        return false;
    }
}

template<>
inline bool to_scalar<string>(string s, string &t) {
    t = s;
    return true;
}

template<>
inline bool to_scalar<bool>(string s, bool &b) {
    if (s == "true" || s == "yes") {
        b = true;
        return true;
    } else if (s == "false" || s == "no") {
        b = false;
        return true;
    } else {
        istringstream i(s);
        if (i >> b) {
            return true;
        } else {
            return false;
        }
    }
}

string to_human_readable(const void *data, int length);
inline string to_human_readable(const string& s) {
    return to_human_readable(s.data(), s.length());
}

string to_hex_string(const void *data, int length);
inline string to_hex_string(const string& s) {
    return to_hex_string(s.data(), s.length());
}

template <typename Int>
inline string to_hex_string(Int l) {
    bool written = false;
    string out = "0x";

    for (int shift = (sizeof(Int) - 1) * 8; shift >= 0; shift -= 8) {
        unsigned char ch = (unsigned char)(l >> shift);
        if (ch) written = true;
        if (written) {
            out += "0123456789ABCDEF"[ch >> 4];
            out += "0123456789ABCDEF"[ch & 0xF];
        }
    }

    if (!written) out += "0";
    return out;
}

string to_escaped_string(const void *data, int length);
inline string to_escaped_string(const string& s) {
    return to_escaped_string(s.data(), s.length());
}

template<typename Array>
inline int array_size(const Array& a) {
    return sizeof(a) / sizeof(a[0]);
}

template<typename Map, typename Key>
inline typename Map::mapped_type lookup(const Map& m, const Key& k) {
    typename Map::const_iterator i = m.find(k);
    if (i == m.end())
        return typename Map::mapped_type();

    return i->second;
}

template<class T1, class T2>
inline pair<T2,T1> reverse_pair(const pair<T1,T2>& p) {
    return pair<T2,T1>(p.second, p.first);
}

template<typename Sequence, typename Item>
inline void erase_in(Sequence& seq, const Item& item)
{
    seq.erase(remove(seq.begin(), seq.end(), item), seq.end());
}

NMSTL_NAMESPACE_END;




#ifdef NMSTL_DMALLOC

#include "dmalloc.h"
#include <new>

inline void* operator new(std::size_t bytes, const char *file, unsigned int line) throw(std::bad_alloc) {
    return _malloc_leap(file, line, bytes);
}
inline void* operator new[](std::size_t bytes, const char *file, unsigned int line) throw(std::bad_alloc) {
    return _malloc_leap(file, line, bytes);
}

inline void operator delete(void *pnt) { _free_leap(0, 0, pnt); }
inline void operator delete[](void *pnt) { _free_leap(0, 0, pnt); }

#define new new(__FILE__, __LINE__)

#endif

#endif
