import horde_api
import sys

SESSION_NAME = "file"
STREAM_NAME = "data"
ADU_BLOCK_SIZE = 2048

total_io = 0
lost = 0
send_rate = 0
srtt = -1

stat_counter = 0
stat_num = 0

def rate_to_kbps(rate):
    kbps = (rate * 8.0) / 1024;
    return "%.1f kbps" % kbps

#
# client
#

def do_client(filename, server):
    if (horde_api.session_connect(server, SESSION_NAME)):
        print "connected to server '%s'." % server
        hstream = horde_api.stream_connect(STREAM_NAME)
        if (hstream > 0):            
            print "stream '%s' connected, uid:%d" % (STREAM_NAME, hstream)

            outfile = open(filename, "wb")

            # define callbacks
            def client_recv_cb(stream_handle, adu_id, totlen, offset, data):
                global total_io;
                
                # save it to disk
                pos = adu_id * ADU_BLOCK_SIZE + offset
                outfile.seek(pos)
                outfile.write(data)
                
                print "saved fragment [adu=%d, offset=%d, len=%d] at %d" % (adu_id,offset,len(data),pos)

                total_io += len(data);

            def stat_cb(total_send, total_recv, total_allowed_send, nchannels):
                global stat_counter, stat_num
                stat_num += 1
                stat_counter += total_recv
                
                tio = total_io / 1024.0
                print "\rstatistics: total_io=%.1fK, send=%d (%s), recv=%d (%s), allowed=%d (%s), nchannels=%d"\
                      % (tio, total_send, rate_to_kbps(total_send), total_recv, rate_to_kbps(total_recv),
                         total_allowed_send, rate_to_kbps(total_allowed_send), nchannels),
                print # debug
                sys.stdout.flush()

            def client_session_cb(event):
                if (event == horde_api.EVENT_SESSION_CLOSED):
                    print "session closed. exiting."
                    horde_api.session_close(SESSION_NAME)
                    outfile.close()
                    sys.exit(0)
                else:
                    print "unknown session event:", event
            
            # register callbacks
            horde_api.session_set_callback(client_session_cb)
            horde_api.stream_set_recv_callback(hstream, client_recv_cb)

            horde_api.statistics_set_callback(stat_cb)
            horde_api.statistics_enable(1)
            
            # enter event loop
            try:
                while (horde_api.event_dispatch() > 0): pass
            except KeyboardInterrupt:
                pass
            
            sys.stderr.write("\nOverall average recv rate: %s\n" % rate_to_kbps(float(stat_counter) / stat_num))
                
#
# server
#
    
def do_server(filename):
    global acked, total_io
    if (horde_api.session_accept(SESSION_NAME)):
        print "client connected."
        hstream = horde_api.stream_accept(STREAM_NAME)
        if (hstream > 0):
            print "stream '%s' connected, uid:%d" % (STREAM_NAME, hstream)
            
            infile = open(filename, "rb")
            infile.seek(0, 2)
            filelen = infile.tell()
            acked = 0

            # define callbacks etc
            def server_session_cb(event):
                if (event == horde_api.EVENT_SESSION_CLOSED):
                    print "session closed. exiting."
                    infile.close()
                    sys.exit(0)
                else:
                    print "unknown session event:", event

            def resend_data(adu_id, offset, fraglen):
                global total_io
                pos = adu_id *  ADU_BLOCK_SIZE + offset
                infile.seek(pos)
                data = infile.read(fraglen)

                horde_api.stream_send_fragment(hstream, adu_id, offset, ADU_BLOCK_SIZE, data)
                total_io += len(data)

            def send_adu(adu_id):
                global total_io
                pos = adu_id *  ADU_BLOCK_SIZE
                infile.seek(pos)
                data = infile.read(ADU_BLOCK_SIZE)

                horde_api.stream_send(hstream, adu_id, data)
                total_io += len(data)

            def stat_cb(total_send, total_recv, total_allowed_send, nchannels):
                global send_rate
                global stat_counter, stat_num
                stat_num += 1
                stat_counter += total_send
                
                tio = total_io / 1024.0
                send_rate = total_send;
                print "statistics: total_io=%.1fK, send=%d (%s), recv=%d (%s), allowed=%d (%s), nchannels=%d, lost=%.1fK, ack=%.1fK, sum=%.1fK"\
                      % (tio, total_send, rate_to_kbps(total_send), total_recv, rate_to_kbps(total_recv),
                         total_allowed_send, rate_to_kbps(total_allowed_send), nchannels,
                         lost/1024.0, acked/1024.0, (lost+acked)/1024.0)
                sys.stdout.flush()


            def server_event_cb(event, stream_handle, a, b, c, d):
                global acked, lost, srtt
                if (event == horde_api.EVENT_ADU_FRAG_ACK):
                    print "ack: adu=%d, offset=%d, len=%d, rtt=%d" % (a,b,c,d)
                    acked += c
                    if (acked >= filelen):
                        # all done
                        print "all done."
                        horde_api.stream_close(STREAM_NAME, hstream)
                        horde_api.session_close(SESSION_NAME)

                    if (srtt < 0): srtt = d
                    else: srtt = (d + 7*srtt)/8
                    
                elif (event == horde_api.EVENT_ADU_FRAG_LOSS):
                    # retransmit
                    print "\tresending data: adu=%d, offset=%d, len=%d" % (a,b,c)
                    resend_data(a, b, c)
                    lost += c
                    
                elif (event == horde_api.EVENT_STREAM_FLUSHED):
                    print "stream flushed:", stream_handle
                    
                elif (event == horde_api.EVENT_STREAM_CLOSED):
                    print "stream closed:", stream_handle
                    
                else:        
                    print "Unknown stream event [%d] on stream %d, a=%d, b=%d, c=%d, d=%d" % (event, stream_handle, a, b, c, d)
                        
            # register callbacks
            horde_api.session_set_callback(server_session_cb)
            horde_api.stream_set_event_callback(hstream, server_event_cb)

            horde_api.statistics_set_callback(stat_cb)
            horde_api.statistics_enable(1)

            try:
                # initially send all the data
                # - event callbacks should notice losses and recover from them
                print "sending all data...",
                
                adu = 0; pos = 0;            
                while (pos <= filelen):
                    send_adu(adu)
                    adu += 1
                    pos += ADU_BLOCK_SIZE

                    while (horde_api.event_dispatch(0) > 0): pass    

                    sys.stderr.write("\rsent: %.1f of %.1f, lost: %.1f (%3.2f%%), rate: %s, srtt: %d    " % \
                        (pos / 1024.0, filelen/1024.0, lost / 1024.0, 100 * float(lost) / total_io, rate_to_kbps(send_rate), srtt))
                    sys.stderr.flush()
                print
                
                # enter event loop
                while (horde_api.event_dispatch() > 0): pass
            except KeyboardInterrupt:
                pass
            sys.stderr.write("\nOverall average send rate: %s\n" % rate_to_kbps(float(stat_counter) / stat_num))
            
if len(sys.argv) == 2:
    # server
    do_server(sys.argv[1])
elif len(sys.argv) == 3:
    # client
    do_client(sys.argv[1], sys.argv[2])
else:
    print "usage:", sys.argv[0], "<local filename> [source server address]"
    print "   server sends from local file; client saves to local file."
