#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "horde.h"

int getch() {
  return fgetc(stdin);
}

void recv_cb(int stream_uid, int adu_id, int total_len, int frag_offset, int frag_len, const char *data, void *context) {
  printf("RECV: stream=%d, adu_id=%d, total_len=%d, frag_start=%d, frag_len=%d\n", stream_uid, adu_id, total_len, frag_offset, frag_len);
  printf("\tDATA: %s\n", to_hexstr(data, frag_len));
}

void event_cb(horde_stream_event event, int stream_uid, int a, int b, int c, int d, void *context) {
  switch (event) {
  case ADU_FRAG_ACK:
    printf("EVENT: code=ADU_FRAG_ACK(%d), stream=%d, adu_id=%d, frag_start=%d, frag_len=%d, rtt=%d\n", (int)event, stream_uid, a, b, c, d);
    break;
  case ADU_FRAG_LOSS:
    printf("EVENT: code=ADU_FRAG_LOSS(%d), stream=%d, adu_id=%d, frag_start=%d, frag_len=%d\n", (int)event, stream_uid, a, b, c);
    break;
  case STREAM_CLOSED:
    printf("EVENT: code=STREAM_CLOSED(%d), stream=%d\n", (int)event, stream_uid);
    break;
  case STREAM_THROTTLE:
  default:
    printf("EVENT: code=%d, stream=%d, a=%d, b=%d, c=%d, d=%d\n", (int)event, stream_uid, a, b, c, d);
  }
}

void session_cb(horde_session_event evcode) { 
  printf("SESSION: event=%d\n", (int)evcode);
}


int main(int argc, char ** argv) {
  horde_set_log_level(LOG_LEVEL_DEBUG);

  if (argc == 2) { // server
    if (horde_session_accept(argv[1])) {
      printf("connected. waiting for stream 'data'...\n");

      int h = horde_stream_accept("data");

      horde_session_set_callback(session_cb);
      horde_stream_set_event_callback(h, event_cb, NULL);
      horde_stream_set_recv_callback(h, recv_cb, NULL);

      printf("handle: %d\nentering event loop.\n", h);

      while(horde_event_dispatch() != -1);

      printf("exiting.\n", h);
      //getch();
      horde_session_close(argv[1]);
    } else {
      printf("session accept failed.\n");
    }
  } else if (argc == 3) { // client
    if (horde_session_connect(argv[2], argv[1])) {
      printf("connected. press a key to open a stream.\n");
      getch();

      int h = horde_stream_connect("data");
      printf("handle: %d\npress a key to terminate.\n", h);
      getch();
      if (h > 0) horde_stream_close("data", h);
      horde_session_close(argv[1]);      
    } else {
      printf("session accept failed.\n");
    }
  } else {
    printf("correct usage: %s <session_id> [peer address]\n", argv[0]);
  }
}
